% Calculates the Black Scholes Price
% By Fabrice Douglas Rouah
function y = BSPrice(S, K, r, T, vol, q, OpType);

% S = Spot Price
% K = Strike Price
% r = risk free rate
% T = Maturity in years
% v = volatility
% q = continuous dividend yield
% OpType = 'C' Call or 'P' put

d1 = (log(S/K) + (r-q+vol^2/2)*T)/vol/sqrt(T);
d2 = d1 - vol*sqrt(T);

if strcmp(OpType,'C')
    y = S*exp(-q*T)*cdf('norm', d1,0,1) - K*exp(-r*T)*cdf('norm', d2,0,1);
else
    y = K*exp(-r*T)*cdf('norm',-d2,0,1) - S*exp(-q*T)*cdf('norm',-d1,0,1);
end;
    

